@@ -1,4 +1,4 @@
-# jhbuild - a build script for GNOME 1.x and 2.x
+# jhbuild - a tool to ease building collections of source packages
 # Copyright (C) 2001-2006  James Henstridge
 #
 #   base.py: the most common jhbuild commands
@@ -18,6 +18,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 import os
+import optparse
 import re
 import stat
 import sys
@@ -28,7 +29,7 @@ import logging
 import jhbuild.moduleset
 import jhbuild.frontends
 from jhbuild.errors import UsageError, FatalError, CommandError
-from jhbuild.commands import Command, register_command
+from jhbuild.commands import Command, BuildCommand, register_command
 
 from jhbuild.config import parse_relative_time
 
@@ -46,10 +47,10 @@ class cmd_update(Command):
                         help=_('treat the given modules as up to date')),
             make_option('-t', '--start-at', metavar='MODULE',
                         action='store', dest='startat', default=None,
-                        help=_('start building at the given module')),
+                        help=_('start updating at the given module')),
             make_option('--tags',
                         action='append', dest='tags', default=[],
-                        help=_('build only modules with the given tags')),
+                        help=_('update only modules with the given tags')),
             make_option('-D', metavar='DATE-SPEC',
                         action='store', dest='sticky_date', default=None,
                         help=_('set a sticky date when checking out modules')),
@@ -63,7 +64,7 @@ class cmd_update(Command):
         module_set = jhbuild.moduleset.load(config)
         module_list = module_set.get_module_list(args or config.modules,
                 config.skip, tags=config.tags,
-                ignore_suggests=config.ignore_suggests)
+                include_suggests=not config.ignore_suggests)
         # remove modules up to startat
         if options.startat:
             while module_list and module_list[0].name != options.startat:
@@ -75,7 +76,7 @@ class cmd_update(Command):
         config.build_targets = ['checkout']
         config.nonetwork = False
 
-        build = jhbuild.frontends.get_buildscript(config, module_list)
+        build = jhbuild.frontends.get_buildscript(config, module_list, module_set=module_set)
         return build.build()
 
 register_command(cmd_update)
@@ -109,7 +110,7 @@ class cmd_updateone(Command):
         config.build_targets = ['checkout']
         config.nonetwork = False
 
-        build = jhbuild.frontends.get_buildscript(config, module_list)
+        build = jhbuild.frontends.get_buildscript(config, module_list, module_set=module_set)
         return build.build()
 
 register_command(cmd_updateone)
@@ -126,6 +127,9 @@ class cmd_cleanone(Command):
             make_option('--honour-config',
                         action='store_true', dest='honour_config', default=False,
                         help=_('honour the makeclean setting in config file')),
+            make_option('--distclean',
+                        action='store_true', dest='distclean', default=False,
+                        help=_('completely clean source tree')),
             ])
 
     def run(self, config, options, args, help=None):
@@ -145,62 +149,17 @@ class cmd_cleanone(Command):
                     _('clean command called while makeclean is set to False, skipped.'))
             return 0
 
-        build = jhbuild.frontends.get_buildscript(config, module_list)
-        return build.build(phases=['clean'])
+        build = jhbuild.frontends.get_buildscript(config, module_list, module_set=module_set)
+        if options.distclean:
+            clean_phase = 'distclean'
+        else:
+            clean_phase = 'clean'
+        return build.build(phases=[clean_phase])
 
 register_command(cmd_cleanone)
 
 
-def check_bootstrap_updateness(config):
-    '''Check install date of bootstrap modules, and compare them to
-       the bootstrap moduleset file last modification date.
-    '''
-    try:
-        module_set = jhbuild.moduleset.load(config, uri = 'bootstrap')
-    except:
-        # failed to get bootstrap moduleset, silently ignore.
-        return
-    packagedb = jhbuild.frontends.get_buildscript(config, []).packagedb
-
-    max_install_date = max([
-            packagedb.installdate(module.name)
-            for module in module_set.modules.values()])
-
-    if max_install_date is None:
-        # bootstrap module has never been built; probably the user doesn't want
-        # to use it
-        return
-
-    updated_modules = []
-    for module in module_set.modules.values():
-        if not packagedb.entries.has_key(module.name):
-            continue
-        p_version = packagedb.entries.get(module.name)[0]
-        if p_version != module.get_revision():
-            updated_modules.append(module.name)
-
-    if not config.modulesets_dir:
-        return
-    bootstrap_uri = os.path.join(config.modulesets_dir, 'bootstrap.modules')
-    if not os.path.exists(bootstrap_uri):
-        return
-    bootstrap_mtime = os.stat(bootstrap_uri)[stat.ST_MTIME]
-
-    if max_install_date <= bootstrap_mtime:
-        # general note, to cover added modules
-        logging.info(
-                _('bootstrap moduleset has been updated since the last time '\
-                  'you used it, perhaps you should run jhbuild bootstrap.'))
-
-    if updated_modules:
-        # note about updated modules
-        logging.info(
-                _('some bootstrap modules have been updated, '\
-                  'perhaps you should update them: %s.') % \
-                  ', '.join(updated_modules))
-
-
-class cmd_build(Command):
+class cmd_build(BuildCommand):
     doc = N_('Update and compile all modules (the default)')
 
     name = 'build'
@@ -209,8 +168,11 @@ class cmd_build(Command):
     def __init__(self):
         Command.__init__(self, [
             make_option('-a', '--autogen',
-                        action='store_true', dest='autogen', default=False,
-                        help=_('always run autogen.sh')),
+                        action='store_true', dest='_unused', default=False,
+                        help=optparse.SUPPRESS_HELP), # no longer used
+            make_option('', '--distclean',
+                        action='store_true', dest='distclean', default=False,
+                        help=_('completely clean source tree')),
             make_option('-c', '--clean',
                         action='store_true', dest='clean', default=False,
                         help=_('run make clean before make')),
@@ -262,20 +224,23 @@ class cmd_build(Command):
             make_option('--min-age', metavar='TIME-SPEC',
                         action='store', dest='min_age', default=None,
                         help=_('skip modules installed less than the given time ago')),
+            make_option('--nodeps',
+                        action='store_false', dest='check_sysdeps', default=None,
+                        help=_('ignore missing system dependencies')),
             ])
 
     def run(self, config, options, args, help=None):
         config.set_from_cmdline_options(options)
 
-        if not config.quiet_mode:
-            check_bootstrap_updateness(config)
-
         module_set = jhbuild.moduleset.load(config)
         modules = args or config.modules
-        module_list = module_set.get_module_list(modules,
-                config.skip, tags = config.tags,
-                include_optional_modules=options.build_optional_modules,
-                ignore_suggests=config.ignore_suggests)
+        full_module_list = module_set.get_full_module_list \
+                               (modules, config.skip,
+                                include_suggests=not config.ignore_suggests,
+                                include_afters=options.build_optional_modules)
+        full_module_list = module_set.remove_tag_modules(full_module_list,
+                                                         config.tags)
+        module_list = module_set.remove_system_modules(full_module_list)
         # remove modules up to startat
         if options.startat:
             while module_list and module_list[0].name != options.startat:
@@ -288,13 +253,24 @@ class cmd_build(Command):
                     _('requested module is in the ignore list, nothing to do.'))
             return 0
 
-        build = jhbuild.frontends.get_buildscript(config, module_list)
+        if config.check_sysdeps:
+            module_state = module_set.get_module_state(full_module_list)
+            if not self.required_system_dependencies_installed(module_state):
+                self.print_system_dependencies(module_state)
+                raise FatalError(_('Required system dependencies not installed.'
+                                   ' Install using the command %(cmd)s or to '
+                                   'ignore system dependencies use command-line'
+                                   ' option %(opt)s' \
+                                   % {'cmd' : "'jhbuild sysdeps --install'",
+                                      'opt' : '--nodeps'}))
+
+        build = jhbuild.frontends.get_buildscript(config, module_list, module_set=module_set)
         return build.build()
 
 register_command(cmd_build)
 
 
-class cmd_buildone(Command):
+class cmd_buildone(BuildCommand):
     doc = N_('Update and compile one or more modules')
 
     name = 'buildone'
@@ -303,11 +279,14 @@ class cmd_buildone(Command):
     def __init__(self):
         Command.__init__(self, [
             make_option('-a', '--autogen',
-                        action='store_true', dest='autogen', default=False,
-                        help=_('always run autogen.sh')),
+                        action='store_true', dest='_unused', default=False,
+                        help=optparse.SUPPRESS_HELP), # no longer used
             make_option('-c', '--clean',
                         action='store_true', dest='clean', default=False,
                         help=_('run make clean before make')),
+            make_option('', '--distclean',
+                        action='store_true', dest='distclean', default=False,
+                        help=_('completely clean source tree')),
             make_option('--check',
                         action='store_true', dest='check', default=False,
                         help=_('run make check after building')),
@@ -340,9 +319,6 @@ class cmd_buildone(Command):
     def run(self, config, options, args, help=None):
         config.set_from_cmdline_options(options)
 
-        if not config.quiet_mode:
-            check_bootstrap_updateness(config)
-
         module_set = jhbuild.moduleset.load(config)
         module_list = []
         for modname in args:
@@ -355,19 +331,18 @@ class cmd_buildone(Command):
                 from jhbuild.modtypes.autotools import AutogenModule
                 module = AutogenModule(modname, default_repo.branch(modname))
                 module.config = config
-                logging.info(_('module "%s" does not exist, created automatically using repository "%s"') % \
-                                (modname, default_repo.name))
+                logging.info(_('module "%(modname)s" does not exist, created automatically using repository "%(reponame)s"') % \
+                             {'modname': modname, 'reponame': default_repo.name})
             module_list.append(module)
 
         if not module_list:
             self.parser.error(_('This command requires a module parameter.'))
 
-        build = jhbuild.frontends.get_buildscript(config, module_list)
+        build = jhbuild.frontends.get_buildscript(config, module_list, module_set=module_set)
         return build.build()
 
 register_command(cmd_buildone)
 
-
 class cmd_run(Command):
     doc = N_('Run a command under the JHBuild environment')
 
@@ -406,7 +381,7 @@ class cmd_run(Command):
             except KeyError, e:
                 raise FatalError(_("A module called '%s' could not be found.") % e)
 
-            build = jhbuild.frontends.get_buildscript(config, module_list)
+            build = jhbuild.frontends.get_buildscript(config, module_list, module_set=module_set)
             builddir = module_list[0].get_builddir(build)
             try:
                 build.execute(args, cwd=builddir)
@@ -423,7 +398,7 @@ class cmd_run(Command):
             except KeyError, e:
                 raise FatalError(_("A module called '%s' could not be found.") % e)
 
-            build = jhbuild.frontends.get_buildscript(config, module_list)
+            build = jhbuild.frontends.get_buildscript(config, module_list, module_set=module_set)
             checkoutdir = module_list[0].get_srcdir(build)
             try:
                 build.execute(args, cwd=checkoutdir)
@@ -494,17 +469,16 @@ class cmd_list(Command):
         config.set_from_cmdline_options(options)
         module_set = jhbuild.moduleset.load(config)
         if options.startat and options.list_all_modules:
-            raise UsageError(_('Conflicting options specified '
-                               '(\'%s\' and \'%s\')') % \
-                               ('--start-at', '--all-modules'))
+            raise UsageError(_('Conflicting options specified (\'--start-at\' and \'--all-modules\')'))
 
         if options.list_all_modules:
             module_list = module_set.modules.values()
         else:
-            module_list = module_set.get_module_list(args or config.modules,
-                                config.skip, tags = config.tags,
-                                include_optional_modules = options.list_optional_modules,
-                                ignore_suggests=config.ignore_suggests)
+            module_list = module_set.get_module_list \
+                              (args or config.modules, config.skip,
+                               tags=config.tags,
+                               include_suggests= not config.ignore_suggests,
+                               include_afters=options.list_optional_modules)
 
         # remove modules up to startat
         if options.startat:
