@@ -1,4 +1,4 @@
-# jhbuild - a build script for GNOME 1.x and 2.x
+# jhbuild - a tool to ease building collections of source packages
 # Copyright (C) 2001-2006  James Henstridge
 # Copyright (C) 2003-2004  Seth Nickell
 #
@@ -32,7 +32,7 @@ class cmd_gui(Command):
     name = 'gui'
     usage_args = ''
 
-    def run(self, config, options, args):
+    def run(self, config, options, args, help=None):
         # request GTK build script.
         config.buildscript = 'gtkui'
 
