@@ -1,4 +1,4 @@
-# jhbuild - a build script for GNOME 1.x and 2.x
+# jhbuild - a tool to ease building collections of source packages
 # Copyright (C) 2001-2006  James Henstridge
 #
 #   info.py: show information about a module
@@ -27,7 +27,6 @@ from jhbuild.commands import Command, register_command
 from jhbuild.modtypes import MetaModule
 from jhbuild.versioncontrol.cvs import CVSBranch
 from jhbuild.versioncontrol.svn import SubversionBranch
-from jhbuild.versioncontrol.arch import ArchBranch
 from jhbuild.versioncontrol.darcs import DarcsBranch
 from jhbuild.versioncontrol.git import GitBranch
 from jhbuild.versioncontrol.tarball import TarballBranch
@@ -40,9 +39,9 @@ class cmd_info(Command):
     usage_args = N_('[ modules ... ]')
 
 
-    def run(self, config, options, args):
-        packagedb = jhbuild.frontends.get_buildscript(config, []).packagedb
+    def run(self, config, options, args, help=None):
         module_set = jhbuild.moduleset.load(config)
+        packagedb = module_set.packagedb
 
         if args:
             for modname in args:
@@ -56,33 +55,41 @@ class cmd_info(Command):
                 self.show_info(module, packagedb, module_set)
 
     def show_info(self, module, packagedb, module_set):
-        installdate = packagedb.installdate(module.name, module.get_revision() or '')
+        package_entry = packagedb.get(module.name)
 
         uprint(_('Name:'), module.name)
         uprint(_('Module Set:'), module.moduleset_name)
         uprint(_('Type:'), module.type)
 
-        if installdate is not None:
+        if package_entry is not None:
+            uprint(_('Install version:'), package_entry.version)
             uprint(_('Install date:'), time.strftime('%Y-%m-%d %H:%M:%S',
-                                                 time.localtime(installdate)))
+                                                     time.localtime(packagedb.installdate(module.name))))
         else:
+            uprint(_('Install version:'), _('not installed'))
             uprint(_('Install date:'), _('not installed'))
 
         if isinstance(module, MetaModule):
             pass
         elif isinstance(module.branch, CVSBranch):
-            uprint(_('CVS-Root:'), module.branch.repository.cvsroot)
-            uprint(_('CVS-Module:'), module.branch.module)
+            uprint(_('CVS Root:'), module.branch.repository.cvsroot)
+            uprint(_('CVS Module:'), module.branch.module)
             if module.branch.revision:
-                uprint(_('CVS-Revision:'), module.branch.revision)
+                uprint(_('CVS Revision:'), module.branch.revision)
         elif isinstance(module.branch, SubversionBranch):
-            uprint(_('Subversion-Module:'), module.branch.module)
-        elif isinstance(module.branch, ArchBranch):
-            uprint(_('Arch-Version:'), module.branch.module)
+            uprint(_('Subversion Module:'), module.branch.module)
         elif isinstance(module.branch, DarcsBranch):
-            uprint(_('Darcs-Archive:'), module.branch.module)
+            uprint(_('Darcs Archive:'), module.branch.module)
         elif isinstance(module.branch, GitBranch):
-            uprint(_('Git-Module:'), module.branch.module)
+            uprint(_('Git Module:'), module.branch.module)
+            if module.branch.unmirrored_module:
+                uprint(_('Git Origin Module:'), module.branch.unmirrored_module)
+            git_branch = module.branch.branch
+            if not git_branch:
+                git_branch = 'master'
+            uprint(_('Git Branch:'), git_branch)
+            if module.branch.tag:
+                uprint(_('Git Tag:'), module.branch.tag)
         elif isinstance(module.branch, TarballBranch):
             uprint(_('URL:'), module.branch.module)
             uprint(_('Version:'), module.branch.version)
@@ -91,6 +98,11 @@ class cmd_info(Command):
             uprint(_('Tree-ID:'), tree_id)
         except (NotImplementedError, AttributeError):
             pass
+        try:
+            source_dir = module.branch.srcdir
+            uprint(_('Sourcedir:'), source_dir)
+        except (NotImplementedError, AttributeError):
+            pass
 
         # dependencies
         if module.dependencies:
@@ -98,7 +110,7 @@ class cmd_info(Command):
         requiredby = [ mod.name for mod in module_set.modules.values()
                        if module.name in mod.dependencies ]
         if requiredby:
-            uprint(_('Required-by:'), ', '.join(requiredby))
+            uprint(_('Required by:'), ', '.join(requiredby))
         if module.suggests:
             uprint(_('Suggests:'), ', '.join(module.suggests))
         if module.after:
