@@ -1,4 +1,4 @@
-# jhbuild - a build script for GNOME 1.x and 2.x
+# jhbuild - a tool to ease building collections of source packages
 # Copyright (C) 2001-2006  James Henstridge
 # Copyright (C) 2007-2008  Frederic Peters
 #
@@ -19,8 +19,11 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 import os
+import os.path
+import re
 import sys
 import traceback
+import time
 import types
 import logging
 import __builtin__
@@ -35,19 +38,19 @@ if sys.platform.startswith('win'):
 __all__ = [ 'Config' ]
 
 _defaults_file = os.path.join(os.path.dirname(__file__), 'defaults.jhbuildrc')
-_default_jhbuildrc = os.path.join(os.environ['HOME'], '.jhbuildrc')
 
 _known_keys = [ 'moduleset', 'modules', 'skip', 'tags', 'prefix',
-                'checkoutroot', 'buildroot', 'autogenargs', 'makeargs',
+                'partial_build', 'checkoutroot', 'buildroot', 'top_builddir',
+                'autogenargs', 'makeargs', 'nice_build', 'jobs',
                 'installprog', 'repos', 'branches', 'noxvfb', 'xvfbargs',
                 'builddir_pattern', 'module_autogenargs', 'module_makeargs',
-                'interact', 'buildscript', 'nonetwork',
-                'alwaysautogen', 'nobuild', 'makeclean', 'makecheck', 'module_makecheck',
+                'interact', 'buildscript', 'nonetwork', 'nobuild',
+                'noinstall', 'makeclean', 'makedistclean', 'makecheck', 'module_makecheck',
                 'use_lib64', 'tinderbox_outputdir', 'sticky_date',
                 'tarballdir', 'pretty_print', 'svn_program', 'makedist',
                 'makedistcheck', 'nonotify', 'notrayicon', 'cvs_program',
                 'checkout_mode', 'copy_dir', 'module_checkout_mode',
-                'build_policy', 'trycheckout', 'min_time',
+                'build_policy', 'trycheckout', 'min_age',
                 'nopoison', 'module_nopoison', 'forcecheck',
                 'makecheck_advisory', 'quiet_mode', 'progress_bar',
                 'module_extra_env', 'jhbuildbot_master', 'jhbuildbot_slavename',
@@ -55,7 +58,11 @@ _known_keys = [ 'moduleset', 'modules', 'skip', 'tags', 'prefix',
                 'jhbuildbot_slaves_dir', 'jhbuildbot_dir',
                 'jhbuildbot_mastercfg', 'use_local_modulesets',
                 'ignore_suggests', 'modulesets_dir', 'mirror_policy',
-                'module_mirror_policy', 'dvcs_mirror_dir', 'build_targets' ]
+                'module_mirror_policy', 'dvcs_mirror_dir', 'build_targets',
+                'cmakeargs', 'module_cmakeargs', 'print_command_pattern',
+                'static_analyzer', 'module_static_analyzer', 'static_analyzer_template', 'static_analyzer_outputdir',
+                'check_sysdeps',
+                ]
 
 env_prepends = {}
 def prependpath(envvar, path):
@@ -105,7 +112,8 @@ def addpath(envvar, path):
             if sys.platform.startswith('win'):
                 path = jhbuild.utils.subprocess_win32.fix_path_for_msys(path)
 
-            if sys.platform.startswith('win') and path[1]==':':
+            if sys.platform.startswith('win') and len(path) > 1 and \
+               path[1] == ':':
                 # Windows: Don't allow c:/ style paths in :-separated env vars
                 # for obvious reasons. /c/ style paths are valid - if a var is
                 # separated by : it will only be of interest to programs inside
@@ -134,13 +142,13 @@ def parse_relative_time(s):
         coeffs = {'s': 1, 'm': 60, 'h': 3600, 'd': 86400, 'w':7*86400}
         return float(m.group(1)) * coeffs[m.group(2)]
     else:
-        raise ValueError(_('unable to parse \'%s\' as relative time.') % s)
+        raise ValueError
 
 
 class Config:
     _orig_environ = None
 
-    def __init__(self, filename=_default_jhbuildrc):
+    def __init__(self, filename):
         self._config = {
             '__file__': _defaults_file,
             'addpath':  addpath,
@@ -150,7 +158,7 @@ class Config:
 
         if not self._orig_environ:
             self.__dict__['_orig_environ'] = os.environ.copy()
-            os.environ['UNMANGLED_PATH'] = os.environ.get('PATH', '')
+        os.environ['UNMANGLED_PATH'] = os.environ.get('PATH', '')
 
         try:
             SRCDIR
@@ -182,12 +190,39 @@ class Config:
         except:
             traceback.print_exc()
             raise FatalError(_('could not load config defaults'))
-        self._config['__file__'] = filename
-        self.filename = filename
-        if not os.path.exists(filename):
-            raise FatalError(_('could not load config file, %s is missing') % filename)
 
-        self.load()
+        old_config = os.path.join(os.path.expanduser('~'), '.jhbuildrc')
+        new_config = os.path.join \
+                         (os.environ.get \
+                             ('XDG_CONFIG_HOME',
+                              os.path.join(os.path.expanduser('~'), '.config')),
+                          'jhbuildrc')
+
+        if filename:
+            if not os.path.exists(filename):
+                raise FatalError(_('could not load config file, %s is missing') % filename)
+        else:
+            if os.path.isfile(old_config) \
+                and not os.path.islink(old_config) \
+                and os.path.isfile(new_config) \
+                and not os.path.islink(new_config):
+                raise FatalError(_('The default location of the configuration '
+                                   'file has changed. Please move %(old_path)s'
+                                   ' to %(new_path)s.' \
+                                   % {'old_path': old_config,
+                                      'new_path': new_config}))
+            if os.path.exists(new_config):
+                filename = new_config
+            elif os.path.exists(old_config):
+                filename = old_config
+
+        if filename:
+            self._config['__file__'] = filename
+            self.filename = filename
+        else:
+            self._config['__file__'] = new_config
+            self.filename = new_config
+        self.load(filename)
         self.setup_env()
 
     def reload(self):
@@ -203,18 +238,19 @@ class Config:
             traceback.print_exc()
             raise FatalError(_('Could not include config file (%s)') % filename)
 
-    def load(self):
+    def load(self, filename=None):
         config = self._config
-        try:
-            execfile(self.filename, config)
-        except Exception, e:
-            if isinstance(e, FatalError):
-                # raise FatalErrors back, as it means an error in include()
-                # and it will print a traceback, and provide a meaningful
-                # message.
-                raise e
-            traceback.print_exc()
-            raise FatalError(_('could not load config file'))
+        if filename:
+            try:
+                execfile(filename, config)
+            except Exception, e:
+                if isinstance(e, FatalError):
+                    # raise FatalErrors back, as it means an error in include()
+                    # and it will print a traceback, and provide a meaningful
+                    # message.
+                    raise e
+                traceback.print_exc()
+                raise FatalError(_('could not load config file'))
 
         if not config.get('quiet_mode'):
             unknown_keys = []
@@ -255,6 +291,15 @@ class Config:
         if config.get('installprog') and os.path.exists(config['installprog']):
             os.environ['INSTALL'] = config['installprog']
 
+        for path_key in ('checkoutroot', 'buildroot', 'top_builddir',
+                         'tinderbox_outputdir', 'tarballdir', 'copy_dir',
+                         'jhbuildbot_slaves_dir', 'jhbuildbot_dir',
+                         'jhbuildbot_mastercfg', 'modulesets_dir',
+                         'dvcs_mirror_dir', 'static_analyzer_outputdir',
+                         'prefix'):
+            if config.get(path_key):
+                config[path_key] = os.path.expanduser(config[path_key])
+
         # copy known config keys to attributes on the instance
         for name in _known_keys:
             setattr(self, name, config[name])
@@ -262,6 +307,10 @@ class Config:
         # default tarballdir to checkoutroot
         if not self.tarballdir: self.tarballdir = self.checkoutroot
 
+        # Ensure top_builddir is absolute
+        if not os.path.isabs(self.top_builddir):
+            self.top_builddir = os.path.join(self.prefix, self.top_builddir)
+
         # check possible checkout_mode values
         seen_copy_mode = (self.checkout_mode == 'copy')
         possible_checkout_modes = ('update', 'clobber', 'export', 'copy')
@@ -282,6 +331,22 @@ class Config:
                 self.use_local_modulesets = False
             self.modulesets_dir = None
 
+        if self.buildroot and not os.path.isabs(self.buildroot):
+            raise FatalError(_('%s must be an absolute path') % 'buildroot')
+        if not os.path.isabs(self.checkoutroot):
+            raise FatalError(_('%s must be an absolute path') % 'checkoutroot')
+        if not os.path.isabs(self.prefix):
+            raise FatalError(_('%s must be an absolute path') % 'prefix')
+        if not os.path.isabs(self.tarballdir):
+            raise FatalError(_('%s must be an absolute path') % 'tarballdir')
+        if (self.tinderbox_outputdir and
+            not os.path.isabs(self.tinderbox_outputdir)):
+            raise FatalError(_('%s must be an absolute path') %
+                             'tinderbox_outputdir')
+
+    def get_original_environment(self):
+        return self._orig_environ
+
     def setup_env(self):
         '''set environment variables for using prefix'''
 
@@ -291,13 +356,28 @@ class Config:
             except:
                 raise FatalError(_('install prefix (%s) can not be created') % self.prefix)
 
+        if not os.path.exists(self.top_builddir):
+            try:
+                os.makedirs(self.top_builddir)
+            except OSError:
+                raise FatalError(
+                        _('working directory (%s) can not be created') % self.top_builddir)
+
+        os.environ['JHBUILD_PREFIX'] = self.prefix
+
         os.environ['UNMANGLED_LD_LIBRARY_PATH'] = os.environ.get('LD_LIBRARY_PATH', '')
 
+        if not os.environ.get('DBUS_SYSTEM_BUS_ADDRESS'):
+            # Use the distribution's D-Bus for the system bus. JHBuild's D-Bus
+            # will # be used for the session bus
+            os.environ['DBUS_SYSTEM_BUS_ADDRESS'] = 'unix:path=/var/run/dbus/system_bus_socket'
+
         # LD_LIBRARY_PATH
         if self.use_lib64:
             libdir = os.path.join(self.prefix, 'lib64')
         else:
             libdir = os.path.join(self.prefix, 'lib')
+        self.libdir = libdir
         addpath('LD_LIBRARY_PATH', libdir)
 
         # LDFLAGS and C_INCLUDE_PATH are required for autoconf configure
@@ -323,15 +403,15 @@ class Config:
 
         # MANPATH
         manpathdir = os.path.join(self.prefix, 'share', 'man')
+        addpath('MANPATH', '')
         addpath('MANPATH', manpathdir)
 
+        # INFOPATH
+        infopathdir = os.path.join(self.prefix, 'share', 'info')
+        addpath('INFOPATH', infopathdir)
+
         # PKG_CONFIG_PATH
-        if os.environ.get('PKG_CONFIG_PATH') is None:
-            # add system pkgconfig lookup-directories by default, as pkg-config
-            # usage spread and is now used by libraries that are out of jhbuild
-            # realm; this also helps when building a single module with
-            # jhbuild.  It is possible to avoid this by setting PKG_CONFIG_PATH
-            # to the empty string.
+        if os.environ.get('PKG_CONFIG_PATH') is None and self.partial_build:
             for dirname in ('share', 'lib', 'lib64'):
                 full_name = '/usr/%s/pkgconfig' % dirname
                 if os.path.exists(full_name):
@@ -341,11 +421,26 @@ class Config:
         addpath('PKG_CONFIG_PATH', pkgconfigdatadir)
         addpath('PKG_CONFIG_PATH', pkgconfigdir)
 
+        # GI_TYPELIB_PATH
+        if not 'GI_TYPELIB_PATH' in os.environ:
+            if self.use_lib64:
+                full_name = '/usr/lib64/girepository-1.0'
+            else:
+                full_name = '/usr/lib/girepository-1.0'
+            if os.path.exists(full_name):
+                addpath('GI_TYPELIB_PATH', full_name)
+        typelibpath = os.path.join(self.libdir, 'girepository-1.0')
+        addpath('GI_TYPELIB_PATH', typelibpath)
+
         # XDG_DATA_DIRS
+        if self.partial_build:
+            addpath('XDG_DATA_DIRS', '/usr/share')
         xdgdatadir = os.path.join(self.prefix, 'share')
         addpath('XDG_DATA_DIRS', xdgdatadir)
 
         # XDG_CONFIG_DIRS
+        if self.partial_build:
+            addpath('XDG_CONFIG_DIRS', '/etc')
         xdgconfigdir = os.path.join(self.prefix, 'etc', 'xdg')
         addpath('XDG_CONFIG_DIRS', xdgconfigdir)
 
@@ -353,6 +448,28 @@ class Config:
         xcursordir = os.path.join(self.prefix, 'share', 'icons')
         addpath('XCURSOR_PATH', xcursordir)
 
+        # GST_PLUGIN_PATH
+        gstplugindir = os.path.join(self.libdir , 'gstreamer-0.10')
+        if os.path.exists(gstplugindir):
+            addpath('GST_PLUGIN_PATH', gstplugindir)
+
+        # GST_PLUGIN_PATH_1_0
+        gstplugindir = os.path.join(self.libdir , 'gstreamer-1.0')
+        if os.path.exists(gstplugindir):
+            addpath('GST_PLUGIN_PATH_1_0', gstplugindir)
+
+        # GST_REGISTRY
+        gstregistry = os.path.join(self.prefix, '_jhbuild', 'gstreamer-0.10.registry')
+        addpath('GST_REGISTRY', gstregistry)
+
+        # GST_REGISTRY_1_0
+        gstregistry = os.path.join(self.prefix, '_jhbuild', 'gstreamer-1.0.registry')
+        addpath('GST_REGISTRY_1_0', gstregistry)
+
+        # ACLOCAL_PATH
+        aclocalpath = os.path.join(self.prefix, 'share', 'aclocal')
+        addpath('ACLOCAL_PATH', aclocalpath)
+
         # ACLOCAL_FLAGS
         aclocaldir = os.path.join(self.prefix, 'share', 'aclocal')
         if not os.path.exists(aclocaldir):
@@ -360,12 +477,22 @@ class Config:
                 os.makedirs(aclocaldir)
             except:
                 raise FatalError(_("Can't create %s directory") % aclocaldir)
+        if self.partial_build:
+            if os.path.exists('/usr/share/aclocal'):
+                addpath('ACLOCAL_FLAGS', '/usr/share/aclocal')
+                if os.path.exists('/usr/local/share/aclocal'):
+                    addpath('ACLOCAL_FLAGS', '/usr/local/share/aclocal')
         addpath('ACLOCAL_FLAGS', aclocaldir)
 
         # PERL5LIB
         perl5lib = os.path.join(self.prefix, 'lib', 'perl5')
         addpath('PERL5LIB', perl5lib)
 
+        # These two variables are so that people who use "jhbuild shell"
+        # can tweak their shell prompts and such to show "I'm under jhbuild".
+        # The first variable is the obvious one to look for; the second
+        # one is for historical reasons. 
+        os.environ['UNDER_JHBUILD'] = 'true'
         os.environ['CERTIFIED_GNOMIE'] = 'yes'
 
         # PYTHONPATH
@@ -375,10 +502,14 @@ class Config:
         python_bin = os.environ.get('PYTHON', 'python')
         try:
             pythonversion = 'python' + get_output([python_bin, '-c',
-                'import sys; print ".".join([str(x) for x in sys.version_info[:2]])'],
+                'import sys; print(".".join([str(x) for x in sys.version_info[:2]]))'],
                 get_stderr = False).strip()
         except CommandError:
             pythonversion = 'python' + str(sys.version_info[0]) + '.' + str(sys.version_info[1])
+            if 'PYTHON' in os.environ:
+                logging.warn(_('Unable to determine python version using the '
+                               'PYTHON environment variable (%s). Using default "%s"')
+                             % (os.environ['PYTHON'], pythonversion))
 
         # In Python 2.6, site-packages got replaced by dist-packages, get the
         # actual value by asking distutils
@@ -386,10 +517,14 @@ class Config:
         try:
             python_packages_dir = get_output([python_bin, '-c',
                 'import os, distutils.sysconfig; '\
-                'print distutils.sysconfig.get_python_lib(prefix="%s").split(os.path.sep)[-1]' % self.prefix],
+                'print(distutils.sysconfig.get_python_lib(prefix="%s").split(os.path.sep)[-1])' % self.prefix],
                 get_stderr=False).strip()
         except CommandError:
             python_packages_dir = 'site-packages'
+            if 'PYTHON' in os.environ:
+                logging.warn(_('Unable to determine python site-packages directory using the '
+                               'PYTHON environment variable (%s). Using default "%s"')
+                             % (os.environ['PYTHON'], python_packages_dir))
             
         if self.use_lib64:
             pythonpath = os.path.join(self.prefix, 'lib64', pythonversion, python_packages_dir)
@@ -465,6 +600,8 @@ class Config:
             self.build_targets.insert(0, 'check')
         if self.makeclean and not 'clean' in self.build_targets:
             self.build_targets.insert(0, 'clean')
+        if self.makedistclean and not 'distclean' in self.build_targets:
+            self.build_targets.insert(0, 'distclean')
         if self.nobuild:
             # nobuild actually means "checkout"
             for phase in ('configure', 'build', 'check', 'clean', 'install'):
@@ -481,14 +618,15 @@ class Config:
             options = self.cmdline_options
         else:
             self.cmdline_options = options
-        if hasattr(options, 'autogen') and options.autogen:
-            self.alwaysautogen = True
-        if hasattr(options, 'clean') and (
-                options.clean and not 'clean' in self.build_targets):
-            self.build_targets.insert(0, 'clean')
         if hasattr(options, 'check') and (
                 options.check and not 'check' in self.build_targets):
             self.build_targets.insert(0, 'check')
+        if hasattr(options, 'clean') and (
+                options.clean and not 'clean' in self.build_targets):
+            self.build_targets.insert(0, 'clean')
+        if hasattr(options, 'distclean') and (
+                options.distclean and not 'distclean' in self.build_targets):
+            self.build_targets.insert(0, 'distclean')
         if hasattr(options, 'dist') and (
                 options.dist and not 'dist' in self.build_targets):
             self.build_targets.append('dist')
@@ -519,16 +657,20 @@ class Config:
             self.build_policy = 'all'
         if hasattr(options, 'min_age') and options.min_age:
             try:
-                self.min_time = time.time() - parse_relative_time(options.min_age)
+                self.min_age = time.time() - parse_relative_time(options.min_age)
             except ValueError:
-                raise FatalError(_('Failed to parse relative time'))
-
+                raise FatalError(_('Failed to parse \'min_age\' relative '
+                                   'time'))
+        if (hasattr(options, 'check_sysdeps') and
+            options.check_sysdeps is not None):
+            self.check_sysdeps = options.check_sysdeps
 
     def __setattr__(self, k, v):
         '''Override __setattr__ for additional checks on some options.'''
         if k == 'quiet_mode' and v:
             try:
                 import curses
+                logging.getLogger().setLevel(logging.ERROR)
             except ImportError:
                 logging.warning(
                         _('quiet mode has been disabled because the Python curses module is missing.'))
