@@ -1,4 +1,4 @@
-# jhbuild - a build script for GNOME 1.x and 2.x
+# jhbuild - a tool to ease building collections of source packages
 # Copyright (C) 2001-2006  James Henstridge
 # Copyright (C) 2003-2004  Seth Nickell
 #
@@ -57,25 +57,25 @@ class AppWindow(gtk.Window, buildscript.BuildScript):
     error_resolution = None
     preference_dialog = None
 
-    def __init__(self, config, module_list=None):
+    def __init__(self, config, module_list=None, module_set=None):
         self.orig_modulelist = module_list
-        buildscript.BuildScript.__init__(self, config)
+        self.module_set = jhbuild.moduleset.load(config)
+        buildscript.BuildScript.__init__(self, config, module_list, module_set=self.module_set)
         self.config = config
         gtk.Window.__init__(self)
         self.set_resizable(False)
         theme = gtk.icon_theme_get_default()
-        gtk.window_set_default_icon_list(
-                theme.load_icon('applications-development', 16, ()),
-                theme.load_icon('applications-development', 24, ()),
-                theme.load_icon('applications-development', 32, ()),
-                theme.load_icon('applications-development', 48, ()),
-                theme.load_icon('applications-development', 64, ()),
-                theme.load_icon('applications-development', 128, ())
-                )
+        if theme.has_icon('applications-development'):
+            gtk.window_set_default_icon_list(
+                    theme.load_icon('applications-development', 16, ()),
+                    theme.load_icon('applications-development', 24, ()),
+                    theme.load_icon('applications-development', 32, ()),
+                    theme.load_icon('applications-development', 48, ()),
+                    theme.load_icon('applications-development', 64, ()),
+                    theme.load_icon('applications-development', 128, ())
+                    )
         self.set_title('JHBuild')
 
-        self.module_set = jhbuild.moduleset.load(config)
-
         self.create_modules_list_model()
         self.create_ui()
         self.notify = notify.Notify(config)
@@ -267,7 +267,7 @@ class AppWindow(gtk.Window, buildscript.BuildScript):
 
     def on_help_cb(self, *args):
         gtk.show_uri(gtk.gdk.screen_get_default(),
-                'ghelp:jhbuild', gtk.get_current_event_time())
+                'help:jhbuild', gtk.get_current_event_time())
 
     def on_preferences_cb(self, *args):
         if not self.preference_dialog:
@@ -282,9 +282,10 @@ class AppWindow(gtk.Window, buildscript.BuildScript):
             modules = [self.modules_list_model.get(
                     self.module_combo.get_active_iter(), 0)[0]]
 
-            self.modulelist = self.module_set.get_module_list(modules,
-                    self.config.skip, tags = self.config.tags,
-                    ignore_suggests=self.config.ignore_suggests)
+            self.modulelist = self.module_set.get_module_list \
+                                  (modules, self.config.skip,
+                                   tags = self.config.tags,
+                                   include_suggests=not self.config.ignore_suggests)
         else:
             self.orig_modulelist = None
 
@@ -348,6 +349,9 @@ class AppWindow(gtk.Window, buildscript.BuildScript):
         self.build_button.set_sensitive(True)
         self.module_hbox.set_sensitive(True)
 
+    def start_phase(self, module, phase):
+        self.notify.clear()
+
     def start_module(self, module):
         idx = [x.name for x in self.modulelist].index(module)
         self.progressbar.set_fraction((1.0*idx) / len(self.modulelist))
@@ -474,6 +478,8 @@ class AppWindow(gtk.Window, buildscript.BuildScript):
                 kws['env'] = os.environ.copy()
                 kws['env'].update(extra_env)
 
+            command = self._prepare_execute(command)
+
             try:
                 p = subprocess.Popen(command, **kws)
             except OSError, e:
@@ -583,7 +589,8 @@ class AppWindow(gtk.Window, buildscript.BuildScript):
                 rc = os.WEXITSTATUS(self.vte_child_exit_status)
             elif os.WIFSIGNALED(self.vte_child_exit_status):
                 raise CommandError(_('%(command)s died with signal %(rc)s') % {
-                        'command': short_command, 'rc': os.WTERMSIG(rc)})
+                        'command': short_command,
+                        'rc': os.WTERMSIG(self.vte_child_exit_status)})
 
         if rc:
             raise CommandError(_('%(command)s returned with an error code (%(rc)s)') % {
@@ -674,7 +681,7 @@ class SelectModulesDialog(gtk.Dialog):
 
         self.startat_model.clear()
         modulelist = self.app.module_set.get_module_list([self.selected_module],
-                ignore_suggests=self.app.config.ignore_suggests)
+                include_suggests=not self.app.config.ignore_suggests)
         for module in modulelist:
             iter = self.startat_model.append((module.name,))
             if module.name == old_start_at:
@@ -711,7 +718,6 @@ class PreferencesDialog(gtk.Dialog):
 
         for key, label in (
                 ('nonetwork', _('Disable network access')),
-                ('alwaysautogen', _('Always run autogen.sh')),
                 ('nopoison', _('Don\'t poison modules on failure'))):
             checkbutton = gtk.CheckButton(label)
             checkbutton.set_active(getattr(self.app.config, key))
