@@ -1,4 +1,4 @@
-# jhbuild - a build script for GNOME 1.x and 2.x
+# jhbuild - a tool to ease building collections of source packages
 # Copyright (C) 2001-2006  James Henstridge
 #
 #   moduleset.py: logic for running the build.
@@ -24,24 +24,47 @@ import sys
 import urlparse
 import logging
 
-from jhbuild.errors import UsageError, FatalError, DependencyCycleError, CommandError
+from jhbuild.errors import UsageError, FatalError, DependencyCycleError, \
+             CommandError, UndefinedRepositoryError
 
 try:
     import xml.dom.minidom
+    import xml.parsers.expat
 except ImportError:
     raise FatalError(_('Python xml packages are required but could not be found'))
 
 from jhbuild import modtypes
 from jhbuild.versioncontrol import get_repo_type
 from jhbuild.utils import httpcache
-from jhbuild.utils.cmds import get_output
+from jhbuild.utils import packagedb
+from jhbuild.utils.cmds import compare_version, get_output
+from jhbuild.modtypes.testmodule import TestModule
+from jhbuild.modtypes.systemmodule import SystemModule
+from jhbuild.versioncontrol.tarball import TarballBranch
+from jhbuild.utils import systeminstall
+from jhbuild.utils import fileutils
 
-__all__ = ['load', 'load_tests']
+__all__ = ['load', 'load_tests', 'get_default_repo']
+
+_default_repo = None
+def get_default_repo():
+    return _default_repo
 
 class ModuleSet:
-    def __init__(self, config = None):
+    def __init__(self, config = None, db=None):
         self.config = config
         self.modules = {}
+        self.raise_exception_on_warning=False
+
+        if db is None:
+            legacy_pkgdb_path = os.path.join(self.config.prefix, 'share', 'jhbuild', 'packagedb.xml')
+            new_pkgdb_path = os.path.join(self.config.top_builddir, 'packagedb.xml')
+            if os.path.isfile(legacy_pkgdb_path):
+                fileutils.rename(legacy_pkgdb_path, new_pkgdb_path)
+            self.packagedb = packagedb.PackageDB(new_pkgdb_path, config)
+        else:
+            self.packagedb = db
+
     def add(self, module):
         '''add a Module object to this set of modules'''
         self.modules[module.name] = module
@@ -49,154 +72,107 @@ class ModuleSet:
     def get_module(self, module_name, ignore_case = False):
         if self.modules.has_key(module_name) or not ignore_case:
             return self.modules[module_name]
-        module_name = module_name.lower()
+        module_name_lower = module_name.lower()
         for module in self.modules.keys():
-            if module.lower() == module_name:
-                if self.config is None or not self.config.quiet_mode:
-                    logging.info(_('fixed case of module \'%(orig)s\' to \'%(new)s\'') % {
-                            'orig': module_name, 'new': module})
+            if module.lower() == module_name_lower:
+                logging.info(_('fixed case of module \'%(orig)s\' to '
+                               '\'%(new)s\'') % {'orig': module_name,
+                                                 'new': module})
                 return self.modules[module]
         raise KeyError(module_name)
 
-    def get_module_list(self, seed, skip=[], tags=[], ignore_cycles=False,
-                ignore_suggests=False, include_optional_modules=False,
-                ignore_missing=False):
-        '''gets a list of module objects (in correct dependency order)
-        needed to build the modules in the seed list'''
+    def get_module_list(self, module_names, skip=[], tags=[],
+                        include_suggests=True, include_afters=False):
+        module_list = self.get_full_module_list(module_names, skip,
+                                                include_suggests,
+                                                include_afters)
+        module_list = self.remove_system_modules(module_list)
+        module_list = self.remove_tag_modules(module_list, tags)
+        return module_list
 
-        if seed == 'all': seed = self.modules.keys()
-        try:
-            all_modules = [self.get_module(mod, ignore_case = True) for mod in seed if mod not in skip]
-        except KeyError, e:
-            raise UsageError(_('module "%s" not found') % e)
+    def get_full_module_list(self, module_names='all', skip=[],
+                                include_suggests=True, include_afters=False,
+                                warn_about_circular_dependencies=False):
 
-        asked_modules = all_modules[:]
-
-        # 1st: get all modules that will be needed
-        # note this is only needed to skip "after" modules that would not
-        # otherwise be built
-        i = 0
-        while i < len(all_modules):
-            for modname in all_modules[i].dependencies:
-                depmod = self.modules.get(modname)
-                if not depmod:
-                    if not ignore_missing:
-                        raise UsageError(_(
-                                '%(module)s has a dependency on unknown "%(invalid)s" module') % {
-                                    'module': all_modules[i].name,
-                                    'invalid': modname})
-                    del all_modules[i]
-                    continue
-                if not depmod in all_modules:
-                    all_modules.append(depmod)
-
-            if not ignore_suggests:
-                # suggests can be ignored if not in moduleset
-                for modname in all_modules[i].suggests:
-                    depmod = self.modules.get(modname)
-                    if not depmod:
-                        continue
-                    if not depmod in all_modules:
-                        all_modules.append(depmod)
-            i += 1
-
-        # 2nd: order them, raise an exception on hard dependency cycle, ignore
-        # them for soft dependencies
-        self._ordered = []
-        self._state = {}
-
-        for modname in skip:
-            # mark skipped modules as already processed
-            self._state[self.modules.get(modname)] = 'processed'
-
-        if tags:
-            for modname in self.modules:
-                for tag in tags:
-                    if tag in self.modules[modname].tags:
+        def dep_resolve(node, resolved, seen, after):
+            ''' Recursive depth-first search of the dependency tree. Creates
+            the build order into the list 'resolved'. <after/> modules are
+            added to the dependency tree but flagged. When search finished
+            <after/> modules not a real dependency are removed.
+            '''
+            circular = False
+            seen.append(node)
+            if include_suggests:
+                edges = node.dependencies + node.suggests + node.after
+            else:
+                edges = node.dependencies + node.after
+            # do not include <after> modules because a previous visited <after>
+            # module may later be a hard dependency
+            resolved_deps = [module for module, after_module in resolved \
+                             if not after_module]
+            for edge_name in edges:
+                edge = self.modules.get(edge_name)
+                if edge == None:
+                    if node not in [i[0] for i in resolved]:
+                        self._warn(_('%(module)s has a dependency on unknown'
+                                     ' "%(invalid)s" module') % \
+                                         {'module'  : node.name,
+                                          'invalid' : edge_name})
+                elif edge_name not in skip and edge not in resolved_deps:
+                    if edge in seen:
+                        # circular dependency detected
+                        circular = True
+                        if self.raise_exception_on_warning:
+                            # Translation of string not required - used in
+                            # unit tests only
+                            raise UsageError('Circular dependencies detected')
+                        if warn_about_circular_dependencies:
+                            self._warn(_('Circular dependencies detected: %s') \
+                                       % ' -> '.join([i.name for i in seen] \
+                                                     + [edge.name]))
                         break
-                else:
-                    # no tag matched, mark module as processed
-                    self._state[self.modules[modname]] = 'processed'
+                    else:
+                        if edge_name in node.after:
+                            dep_resolve(edge, resolved, seen, True)
+                        elif edge_name in node.suggests:
+                            dep_resolve(edge, resolved, seen, after)
+                        elif edge_name in node.dependencies:
+                            dep_resolve(edge, resolved, seen, after)
+                            # hard dependency may be missed if a cyclic
+                            # dependency. Add it:
+                            if edge not in [i[0] for i in resolved]:
+                                resolved.append((edge, after))
 
-        def order(modules, module, mode = 'dependencies'):
-            if self._state.get(module, 'clean') == 'processed':
-                # already seen
-                return
-            if self._state.get(module, 'clean') == 'in-progress':
-                # dependency circle, abort when processing hard dependencies
-                if not ignore_cycles:
-                    raise DependencyCycleError()
-                else:
-                    self._state[module] = 'in-progress'
-                    return
-            self._state[module] = 'in-progress'
-            for modname in module.dependencies:
-                depmod = self.modules[modname]
-                order([self.modules[x] for x in depmod.dependencies], depmod, 'dependencies')
-            if not ignore_suggests:
-                for modname in module.suggests:
-                    depmod = self.modules.get(modname)
-                    if not depmod:
-                        continue
-                    save_state, save_ordered = self._state.copy(), self._ordered[:]
-                    try:
-                        order([self.modules[x] for x in depmod.dependencies], depmod, 'suggests')
-                    except DependencyCycleError:
-                        self._state, self._ordered = save_state, save_ordered
+            seen.remove(node)
 
-            extra_afters = []
-            for modname in module.after:
-                depmod = self.modules.get(modname)
-                if not depmod:
-                    # this module doesn't exist, skip.
-                    continue
-                if not depmod in all_modules and not include_optional_modules:
-                    # skip modules that would not be built otherwise
-                    # (build_optional_modules being the argument to force them
-                    # to be included nevertheless)
+            if not circular:
+                if node not in [i[0] for i in resolved]:
+                    resolved.append((node, after))
+                elif not after:
+                    # a dependency exists for an after, flag to keep
+                    for index, item in enumerate(resolved):
+                        if item[1] == True and item[0] == node:
+                            resolved[index] = (node, False)
 
-                    if not depmod.dependencies:
-                        # depmod itself has no dependencies, skip.
-                        continue
+        if module_names == 'all':
+            module_names = self.modules.keys()
+        try:
+            # remove skip modules from module_name list
+            modules = [self.get_module(module, ignore_case = True) \
+                       for module in module_names if module not in skip]
+        except KeyError, e:
+            raise UsageError(_("A module called '%s' could not be found.") % e)
 
-                    # more expensive, if depmod has dependencies, compute its
-                    # full list of hard dependencies, getting it into
-                    # extra_afters, so they are also evaluated.
-                    # <http://bugzilla.gnome.org/show_bug.cgi?id=546640>
-                    t_ms = ModuleSet(self.config)
-                    t_ms.modules = self.modules.copy()
-                    dep_modules = t_ms.get_module_list(seed=[depmod.name])
-                    for m in dep_modules[:-1]:
-                        if m in all_modules:
-                            extra_afters.append(m)
-                    continue
-                save_state, save_ordered = self._state.copy(), self._ordered[:]
-                try:
-                    order([self.modules[x] for x in depmod.dependencies], depmod, 'after')
-                except DependencyCycleError:
-                    self._state, self._ordered = save_state, save_ordered
-            for depmod in extra_afters:
-                save_state, save_ordered = self._state.copy(), self._ordered[:]
-                try:
-                    order([self.modules[x] for x in depmod.dependencies], depmod, 'after')
-                except DependencyCycleError:
-                    self._state, self._ordered = save_state, save_ordered
-            self._state[module] = 'processed'
-            self._ordered.append(module)
+        resolved = []
+        for module in modules:
+            dep_resolve(module, resolved, [], False)
 
-        for i, module in enumerate(all_modules):
-            order([], module)
-            if i+1 == len(asked_modules): 
-                break
-
-        ordered = self._ordered[:]
-        del self._ordered
-        del self._state
-        return ordered
-    
-    def get_full_module_list(self, skip=[], ignore_cycles=False):
-        return self.get_module_list(self.modules.keys(), skip=skip,
-                ignore_cycles=ignore_cycles, ignore_missing=True)
+        if include_afters:
+            module_list = [module[0] for module in resolved]
+        else:
+            module_list = [module for module, after_module in resolved \
+                           if not after_module]
+        return module_list
 
     def get_test_module_list (self, seed, skip=[]):
         test_modules = []
@@ -207,7 +183,77 @@ class ModuleSet:
                 if test_app in mod.tested_pkgs:
                     test_modules.append(mod)
         return test_modules
-    
+
+    def get_module_state(self, modules):
+        installed_pkgconfig = systeminstall.get_installed_pkgconfigs(self.config)
+        
+        module_state = {}
+        for module in modules:
+            # only consider SystemModules or modules with <pkg-config>
+            if (isinstance(module, SystemModule) or
+                (isinstance(module.branch, TarballBranch) and
+                 module.pkg_config is not None)):
+                required_version = module.branch.version
+                installed_version = None
+                new_enough = False
+                systemmodule = isinstance(module, SystemModule)
+                if module.pkg_config is not None:
+                    # strip off the .pc
+                    module_pkg = module.pkg_config[:-3]
+                    if module_pkg in installed_pkgconfig:
+                        installed_version = installed_pkgconfig[module_pkg]
+                        if required_version is None:
+                            new_enough = True
+                        else:
+                            new_enough = compare_version(installed_version,
+                                                         required_version)
+                elif systemmodule:
+                    new_enough = systeminstall.systemdependencies_met \
+                                     (module.name, module.systemdependencies,
+                                      self.config)
+                    if new_enough:
+                        installed_version = 'unknown'
+                module_state[module] = (required_version, installed_version,
+                                        new_enough, systemmodule)
+        return module_state
+
+    def remove_system_modules(self, modules):
+        if not self.config.partial_build:
+            return [module for module in modules \
+                    if not isinstance(module, SystemModule)]
+
+        return_list = []
+
+        installed_pkgconfig = systeminstall.get_installed_pkgconfigs \
+                                (self.config)
+
+        for module in modules:
+            if isinstance(module, SystemModule):
+                continue
+            skip = False
+            if module.pkg_config is not None and \
+            isinstance(module.branch, TarballBranch):
+                # Strip off the .pc
+                module_pkg = module.pkg_config[:-3]
+                required_version = module.branch.version
+                if module_pkg in installed_pkgconfig:
+                    installed_version = installed_pkgconfig[module_pkg]
+                    skip = compare_version(installed_version, required_version)
+            if not skip:
+                return_list.append(module)
+        return return_list
+
+    def remove_tag_modules(self, modules, tags):
+        if tags:
+            return_list = []
+            for module in modules:
+                for tag in tags:
+                    if tag in self.modules[module.name].tags:
+                        return_list.append(module)
+            return return_list
+        else:
+            return modules
+
     def write_dot(self, modules=None, fp=sys.stdout, suggests=False, clusters=False):
         from jhbuild.modtypes import MetaModule
         from jhbuild.modtypes.autotools import AutogenModule
@@ -273,6 +319,13 @@ class ModuleSet:
 
         fp.write('}\n')
 
+    def _warn(self, msg):
+        if self.raise_exception_on_warning:
+            raise UsageError(msg)
+        else:
+            logging.warn(msg)
+
+
 def load(config, uri=None):
     if uri is not None:
         modulesets = [ uri ]
@@ -282,22 +335,25 @@ def load(config, uri=None):
         modulesets = [ config.moduleset ]
     ms = ModuleSet(config = config)
     for uri in modulesets:
-        if '/' not in uri and not os.path.exists(uri):
-            if config.modulesets_dir and config.nonetwork or config.use_local_modulesets:
+        if os.path.isabs(uri):
+            pass
+        elif config.modulesets_dir and config.nonetwork or config.use_local_modulesets:
+            if os.path.isfile(os.path.join(config.modulesets_dir,
+                                           uri + '.modules')):
                 uri = os.path.join(config.modulesets_dir, uri + '.modules')
-            else:
-                uri = 'http://git.gnome.org/cgit/jhbuild/plain/modulesets/%s.modules' % uri
-        try:
-            ms.modules.update(_parse_module_set(config, uri).modules)
-        except xml.parsers.expat.ExpatError, e:
-            raise FatalError(_('failed to parse %s: %s') % (uri, e))
+            elif os.path.isfile(os.path.join(config.modulesets_dir, uri)):
+                uri = os.path.join(config.modulesets_dir, uri)
+        elif not urlparse.urlparse(uri)[0]:
+            uri = 'http://git.gnome.org/browse/jhbuild/plain/modulesets' \
+                  '/%s.modules' % uri
+        ms.modules.update(_parse_module_set(config, uri).modules)
     return ms
 
 def load_tests (config, uri=None):
     ms = load (config, uri)
     ms_tests = ModuleSet(config = config)
     for app, module in ms.modules.iteritems():
-        if module.__class__ == testmodule.TestModule:
+        if module.__class__ == TestModule:
             ms_tests.modules[app] = module
     return ms_tests
 
@@ -321,12 +377,16 @@ def _parse_module_set(config, uri):
         document = xml.dom.minidom.parse(filename)
     except IOError, e:
         raise FatalError(_('failed to parse %s: %s') % (filename, e))
+    except xml.parsers.expat.ExpatError, e:
+        raise FatalError(_('failed to parse %s: %s') % (uri, e))
 
     assert document.documentElement.nodeName == 'moduleset'
     moduleset = ModuleSet(config = config)
     moduleset_name = document.documentElement.getAttribute('name')
-    if not moduleset_name and uri.endswith('.modules'):
-        moduleset_name = os.path.basename(uri)[:-len('.modules')]    
+    if not moduleset_name:
+        moduleset_name = os.path.basename(uri)
+        if moduleset_name.endswith('.modules'):
+            moduleset_name = moduleset_name[:-len('.modules')]
 
     # load up list of repositories
     repositories = {}
@@ -344,6 +404,8 @@ def _parse_module_set(config, uri):
             for attr in repo_class.init_xml_attrs:
                 if node.hasAttribute(attr):
                     kws[attr.replace('-', '_')] = node.getAttribute(attr)
+            if name in repositories:
+                logging.warning(_('Duplicate repository:') + ' '+ name)
             repositories[name] = repo_class(config, name, **kws)
             repositories[name].moduleset_uri = uri
             mirrors = {}
@@ -383,6 +445,8 @@ def _parse_module_set(config, uri):
             inc_uri = urlparse.urljoin(uri, href)
             try:
                 inc_moduleset = _parse_module_set(config, inc_uri)
+            except UndefinedRepositoryError:
+                raise
             except FatalError, e:
                 if inc_uri[0] == '/':
                     raise e
@@ -401,9 +465,13 @@ def _parse_module_set(config, uri):
             if moduleset_name:
                 module.tags.append(moduleset_name)
             module.moduleset_name = moduleset_name
-            module.config = config
             moduleset.add(module)
 
+    # keep default repository around, used when creating automatic modules
+    global _default_repo
+    if default_repo:
+        _default_repo = repositories[default_repo]
+
     return moduleset
 
 def warn_local_modulesets(config):
@@ -442,5 +510,5 @@ def warn_local_modulesets(config):
     logging.info(
             _('Modulesets were edited locally but JHBuild is configured '\
               'to get them from the network, perhaps you need to add '\
-              'use_local_modulesets = True to your .jhbuildrc.'))
+              'use_local_modulesets = True to your %s.' % config.filename))
 
